from __future__ import annotations

import os

from devenv.constants import pythons_root
from devenv.lib import archive


def get(python_version: str, url: str, sha256: str) -> str:
    unpack_into = f"{pythons_root}/{python_version}"

    if os.path.exists(f"{unpack_into}/python/bin/python3"):
        return f"{unpack_into}/python/bin/python3"

    archive_file = archive.download(url, sha256)
    archive.unpack(archive_file, unpack_into)

    assert os.path.exists(f"{unpack_into}/python/bin/python3")
    return f"{unpack_into}/python/bin/python3"



from __future__ import annotations

from typing import Set
from typing import Tuple

from devenv.lib_check.types import checker

tags: Set[str] = set()


@checker
def check() -> Tuple[bool, str]:
    return True, ""



from __future__ import annotations

import configparser
import os
import shutil
import subprocess
import sys
from collections.abc import Sequence
from typing import Dict
from typing import Tuple

from devenv import constants
from devenv import pythons
from devenv.constants import CI
from devenv.constants import DARWIN
from devenv.constants import home
from devenv.constants import MACHINE
from devenv.constants import SYSTEM_MACHINE
from devenv.constants import VOLTA_HOME
from devenv.lib import colima
from devenv.lib import limactl
from devenv.lib import proc
from devenv.lib import volta

help = "Resyncs the environment."


def run_procs(
    repo: str,
    reporoot: str,
    venv: str,
    _procs: Tuple[Tuple[str, tuple[str, ...]], ...],
) -> bool:
    procs: list[tuple[str, tuple[str, ...], subprocess.Popen[bytes]]] = []

    for name, cmd in _procs:
        print(f"⏳ {name}")
        if constants.DEBUG:
            proc.xtrace(cmd)
        procs.append(
            (
                name,
                cmd,
                subprocess.Popen(
                    cmd,
                    stdout=subprocess.PIPE,
                    env={
                        **constants.user_environ,
                        **proc.base_env,
                        "VIRTUAL_ENV": venv,
                        "VOLTA_HOME": VOLTA_HOME,
                        "PATH": f"{venv}/bin:{proc.base_path}",
                    },
                    cwd=reporoot,
                ),
            )
        )

    all_good = True
    for name, final_cmd, p in procs:
        p.wait()
        if p.returncode != 0:
            all_good = False
            if p.stdout is None:
                out = ""
            else:
                out = p.stdout.read().decode()
            print(
                f"""
❌ {name}

failed command (code p.returncode):
    {proc.quote(final_cmd)}

Output:
{out}

"""
            )
        else:
            print(f"✅ {name}")

    return all_good


def main(context: Dict[str, str], argv: Sequence[str] | None = None) -> int:
    repo = context["repo"]
    if repo not in {"sentry", "getsentry"}:
        print(f"repo {repo} not supported yet!")
        return 1

    reporoot = context["reporoot"]

    repo_config = configparser.ConfigParser()
    repo_config.read(f"{reporoot}/devenv/config.ini")

    python_version = repo_config["python"]["version"]
    url = repo_config["python"][f"{sys.platform}_{MACHINE}"]
    sha256 = repo_config["python"][f"{sys.platform}_{MACHINE}_sha256"]

    venv = f"{reporoot}/.venv"
    if not os.path.exists(venv):
        print(f"virtualenv for {repo} doesn't exist, creating one at {venv}...")
        proc.run(
            (pythons.get(python_version, url, sha256), "-m", "venv", venv),
            exit=True,
        )

    # Check the python version. If mismatch, then recreate the venv.
    # This helps smooth out the python version upgrade experience.
    # XXX: it isn't in a format configparser can read as there are no sections
    venv_version = ""
    with open(f"{venv}/pyvenv.cfg", "r") as f:
        for line in f:
            if line.startswith("version"):
                venv_version = line.split("=")[1].strip()
                break
    if venv_version != python_version:
        print(f"outdated virtualenv version (python {venv_version})!")
        print("creating a new one...")
        shutil.rmtree(venv)
        proc.run(
            (pythons.get(python_version, url, sha256), "-m", "venv", venv),
            exit=True,
        )

    print("Resyncing your dev environment.")

    if not run_procs(
        repo,
        reporoot,
        venv,
        (
            (
                "git and precommit",
                # this can't be done in paralell with python dependencies
                # as multiple pips cannot act on the same venv
                ("make", "setup-git"),
            ),
        ),
    ):
        return 1

    # This is for engineers with existing dev environments transitioning over.
    # Bootstrap will set devenv-managed volta up but they won't be running
    # devenv bootstrap, just installing devenv then running devenv sync.
    # make install-js-dev will fail since our run_procs expects devenv-managed
    # volta.
    volta.install()

    if DARWIN:
        # we don't officially support colima on linux yet
        if CI:
            # colima 0.6.8 doesn't work with macos-13,
            # but integration coverage is still handy
            colima.install(
                "0.6.2",
                "darwin_x86_64",
                "43ef3fc80a8347d51b8ec1706f9caf8863bd8727a6f7532caf1ccd20497d8485",
            )
        else:
            colima.install(
                repo_config["colima"]["version"],
                repo_config["colima"][SYSTEM_MACHINE],
                repo_config["colima"][f"{SYSTEM_MACHINE}_sha256"],
            )

        # TODO: move limactl version into per-repo config
        limactl.install()

    if not run_procs(
        repo,
        reporoot,
        venv,
        (
            ("javascript dependencies", ("make", "install-js-dev")),
            ("python dependencies", ("make", "install-py-dev")),
        ),
    ):
        return 1

    if not os.path.exists(f"{home}/.sentry/config.yml") or not os.path.exists(
        f"{home}/.sentry/sentry.conf.py"
    ):
        proc.run((f"{venv}/bin/sentry", "init", "--dev"))

    # TODO: run devservices healthchecks for redis and postgres to bypass this
    proc.run(
        (f"{venv}/bin/sentry", "devservices", "up", "redis", "postgres"),
        exit=True,
    )

    if run_procs(
        repo,
        reporoot,
        venv,
        (
            (
                "python migrations",
                (f"{venv}/bin/sentry", "upgrade", "--noinput"),
            ),
        ),
    ):
        return 0
    else:
        return 1



from __future__ import annotations

from collections.abc import Callable
from typing import Tuple


def checker(
    f: Callable[[], Tuple[bool, str]]
) -> Callable[[], Tuple[bool, str]]:
    return f


def fixer(f: Callable[[], Tuple[bool, str]]) -> Callable[[], Tuple[bool, str]]:
    return f



from __future__ import annotations

import os
import subprocess
from unittest.mock import call
from unittest.mock import patch

from devenv import main
from tests.utils import chdir


def test(tmp_path: str) -> None:
    coderoot = f"{tmp_path}/coderoot"
    configroot = f"{tmp_path}/configroot"

    with patch("devenv.main.CI", True), patch("sentry_sdk.init"), patch(
        "devenv.main.config_root", configroot
    ), patch("devenv.bootstrap.main") as mock_bootstrap, patch(
        "devenv.sync.main"
    ) as mock_sync:
        main.DEFAULT_CONFIG["devenv"]["coderoot"] = coderoot
        main.devenv(("/path/to/argv0", "bootstrap"))
        with open(f"{configroot}/config.ini", "r") as f:
            assert (
                f.read()
                == f"""[devenv]
# please enter the root directory you want to work in
coderoot = {coderoot}

"""
            )
            assert mock_bootstrap.mock_calls == [call(coderoot, [])]

        # with our config written, let's mock a repo then call sync
        repo = "repo"
        reporoot = f"{coderoot}/repo"
        os.makedirs(reporoot)
        subprocess.run(("git", "init", "--quiet", "--bare", reporoot))
        with chdir(reporoot), patch(
            "os.makedirs", wraps=os.makedirs
        ) as os_makedirs:
            main.devenv(("/path/to/argv0", "sync"))

            # this time, we should have early returned from initialize_config
            # so only the os.makedirs call to coderoot should have happened
            # otherwise we would also have a call(configroot, exist_ok=True)
            assert os_makedirs.mock_calls == [call(coderoot, exist_ok=True)]
            assert mock_sync.mock_calls == [
                call({"repo": repo, "reporoot": reporoot}, [])
            ]



from __future__ import annotations

import os
import shutil
import tempfile

from devenv.constants import bin_root
from devenv.constants import home
from devenv.lib import archive
from devenv.lib import proc


def _install(url: str, sha256: str, into: str) -> None:
    with tempfile.TemporaryDirectory(dir=into) as tmpd:
        archive.download(url, sha256, dest=f"{tmpd}/colima")
        os.replace(f"{tmpd}/colima", f"{into}/colima")
        os.chmod(f"{into}/colima", 0o775)


def uninstall() -> None:
    for d in (f"{home}/.lima",):
        shutil.rmtree(d, ignore_errors=True)

    for f in (f"{bin_root}/colima",):
        if os.path.exists(f):
            os.remove(f)


def install(version: str, url: str, sha256: str) -> None:
    if shutil.which("colima", path=bin_root) == f"{bin_root}/colima":
        stdout = proc.run((f"{bin_root}/colima", "--version"), stdout=True)
        installed_version = stdout.strip().split()[-1]
        if version == installed_version:
            return
        print(f"installed colima {installed_version} is outdated!")

    print(f"installing colima {version}...")
    uninstall()
    _install(url, sha256, bin_root)

    stdout = proc.run((f"{bin_root}/colima", "--version"), stdout=True)
    if f"colima version {version}" not in stdout:
        raise SystemExit("Failed to install colima {version}!")



from __future__ import annotations

import os
import platform
import tempfile
from shutil import which

from devenv.constants import bin_root
from devenv.constants import MACHINE
from devenv.lib import archive

_version = "0.19.1"
_sha256 = {
    f"lima-{_version}-Darwin-arm64.tar.gz": "0dfcf3a39782baf1c2ea43cf026f8df0321c671d914c105fbb78de507aa8bda4",
    f"lima-{_version}-Darwin-x86_64.tar.gz": "ac8827479f66ef1b288b31f164b22f6433faa14c44ce5bbebe09e6e913582479",
    f"lima-{_version}-Linux-aarch64.tar.gz": "c55e57ddbefd9988d0f3676bb873bcc6e0f7b3c3d47a1f07599ee151c5198d96",
    f"lima-{_version}-Linux-x86_64.tar.gz": "7d18b1716aae14bf98d6ea93a703e8877b0c3142f7ba2e87401d47d5d0fe3ff1",
}


class UnexpectedPlatformError(Exception):
    pass


def build_name() -> str | None:
    system = platform.system()
    if system == "Linux":
        if MACHINE == "arm64":
            return f"lima-{_version}-{system}-aarch64.tar.gz"
        return f"lima-{_version}-{system}-{MACHINE}.tar.gz"
    elif system == "Darwin":
        return f"lima-{_version}-{system}-{MACHINE}.tar.gz"
    else:
        raise UnexpectedPlatformError(f"Unexpected OS: {platform.platform()}")


def download_and_unpack_archive(name: str, into: str) -> None:
    url = (
        "https://github.com/lima-vm/lima/releases/download/"
        f"v{_version}/{name}"
    )

    archive_file = archive.download(url, _sha256[name], dest=f"{into}/{name}")
    archive.unpack(archive_file, into)


def _install(into: str) -> None:
    name = build_name()
    if name is None:
        return

    with tempfile.TemporaryDirectory(dir=into) as tmpd:
        download_and_unpack_archive(name, tmpd)

        # the archive was atomically placed into tmpd so
        # these are on the same fs and can be atomically moved too
        os.replace(f"{tmpd}/bin/lima", f"{into}/lima")
        os.replace(f"{tmpd}/bin/limactl", f"{into}/limactl")
        os.replace(
            f"{tmpd}/share/lima/lima-guestagent.Linux-aarch64",
            f"{into}/lima-guestagent.Linux-aarch64",
        )
        os.replace(
            f"{tmpd}/share/lima/lima-guestagent.Linux-x86_64",
            f"{into}/lima-guestagent.Linux-x86_64",
        )
        os.makedirs(f"{into}/templates", exist_ok=True)
        os.replace(
            f"{tmpd}/share/lima/templates/default.yaml",
            f"{into}/templates/default.yaml",
        )


def install() -> None:
    # this needs to be better
    if (
        which("lima", path=bin_root) == f"{bin_root}/lima"
        and which("limactl", path=bin_root) == f"{bin_root}/limactl"
    ):
        return

    _install(bin_root)

    if not os.path.exists(f"{bin_root}/lima") or not os.path.exists(
        f"{bin_root}/limactl"
    ):
        raise SystemExit("Failed to install colima!")



from __future__ import annotations

from typing import Set
from typing import Tuple

from devenv.lib_check.types import checker
from devenv.lib_check.types import fixer

tags: Set[str] = set(["test", "fail"])
name = "failing check"


@checker
def check() -> Tuple[bool, str]:
    return False, ""


@fixer
def fix() -> Tuple[bool, str]:
    return False, ""



from __future__ import annotations

import argparse
import configparser
import os
from collections.abc import Sequence
from typing import cast
from typing import Optional

from typing_extensions import TypeAlias

from devenv import bootstrap
from devenv import doctor
from devenv import pin_gha
from devenv import sync
from devenv.constants import CI
from devenv.constants import config_root
from devenv.lib.fs import gitroot

ExitCode: TypeAlias = "str | int | None"

# comments are used as input prompts for initial config
Config: TypeAlias = "dict[str, dict[str, str | None]]"
DEFAULT_CONFIG = dict(
    devenv={
        "# please enter the root directory you want to work in": None,
        "coderoot": "~/code",
    }
)


def initialize_config(config_path: str, defaults: Config) -> None:
    if os.path.exists(config_path):
        # todo: query for any config options not in  the existing config file
        return

    config = configparser.ConfigParser(allow_no_value=True)
    config.read_dict(defaults)

    if not CI:
        for section, values in config.items():
            for var, _val in values.items():
                if section == "devenv" and var == "coderoot":
                    # this is a special case used to make the transition from existing
                    # dev environments easier as we can guess the desired coderoot if
                    # devenv is run inside of a git repo
                    try:
                        reporoot = gitroot()
                    except RuntimeError:
                        pass
                    else:
                        coderoot = os.path.abspath(f"{reporoot}/..")
                        print(f"\nWe autodetected a coderoot: {coderoot}")
                        config.set(section, var, coderoot)
                        continue

                # typshed doesn't account for `allow_no_value`
                val = cast(Optional[str], _val)
                if val is None:
                    print(var.strip("# "), end="")
                else:
                    try:
                        val = input(f" [{val}]: ") or val
                    except EOFError:
                        # noninterative, use the defaults
                        print()
                    config.set(section, var, val)
    print("Thank you. Saving answers.")
    os.makedirs(config_root, exist_ok=True)
    with open(config_path, "w") as f:
        config.write(f)
    print(f"If you made a mistake, you can edit {config_path}.")


class CustomHelpFormat(
    argparse.RawTextHelpFormatter, argparse.ArgumentDefaultsHelpFormatter
):
    pass


def parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(formatter_class=CustomHelpFormat)
    parser.add_argument(
        "command",
        choices=("bootstrap", "doctor", "sync", "pin-gha"),
        metavar="COMMAND",
        help=f"""\
bootstrap - {bootstrap.help}
doctor    - {doctor.help}
sync      - {sync.help}
pin-gha   - {pin_gha.help}
""",
    )
    parser.add_argument(
        "--nocoderoot",
        action="store_true",
        help="Do not require being in coderoot.",
    )
    return parser


def devenv(argv: Sequence[str]) -> ExitCode:
    args, remainder = parser().parse_known_args(argv[1:])

    # generic/standalone tools that do not care about devenv configuration
    if args.command == "pin-gha":
        return pin_gha.main(remainder)

    config_path = f"{config_root}/config.ini"
    initialize_config(config_path, DEFAULT_CONFIG)

    config = configparser.ConfigParser(allow_no_value=True)
    config.read(config_path)
    coderoot = os.path.expanduser(config["devenv"]["coderoot"])
    os.makedirs(coderoot, exist_ok=True)

    if args.command == "bootstrap":
        return bootstrap.main(coderoot, remainder)

    if not args.nocoderoot and not os.getcwd().startswith(coderoot):
        print(
            f"You aren't in your code root ({coderoot})!\n"
            "To ignore, use devenv --nocoderoot [COMMAND]\n"
            f"To change your code root, you can edit {config_path}.\n"
        )
        return 1

    # the remaining tools are repo-specific
    reporoot = gitroot()
    repo = reporoot.split("/")[-1]

    context = {"repo": repo, "reporoot": reporoot}

    if args.command == "doctor":
        return doctor.main(context, remainder)
    if args.command == "sync":
        return sync.main(context, remainder)

    return 1


def main() -> ExitCode:
    import sys

    import sentry_sdk

    sentry_sdk.init(
        # https://sentry.sentry.io/settings/projects/sentry-dev-env/keys/
        dsn="https://9bdb053cb8274ea69231834d1edeec4c@o1.ingest.sentry.io/5723503",
        # enable performance monitoring
        enable_tracing=True,
    )

    return devenv(sys.argv)


if __name__ == "__main__":
    raise SystemExit(main())



from __future__ import annotations

import os
from unittest.mock import call
from unittest.mock import patch

import pytest

from devenv.lib.volta import _sha256
from devenv.lib.volta import _version
from devenv.lib.volta import install
from devenv.lib.volta import install_volta
from devenv.lib.volta import populate_volta_home_with_shims
from devenv.lib.volta import UnexpectedPlatformError


def test_install(tmp_path: str) -> None:
    VOLTA_HOME = f"{tmp_path}/volta"
    os.makedirs(f"{tmp_path}/bin")
    os.makedirs(f"{VOLTA_HOME}/bin")
    for executable in ("node", "npm", "npx", "yarn", "pnpm"):
        open(f"{VOLTA_HOME}/bin/{executable}", "a").close()

    with patch.multiple(
        "devenv.lib.volta", root=tmp_path, VOLTA_HOME=VOLTA_HOME
    ):
        with patch("devenv.lib.volta.which", side_effect=[None, None]), patch(
            "devenv.lib.volta.install_volta"
        ) as mock_install_volta, patch(
            "devenv.lib.volta.proc.run",
            side_effect=[None, _version],  # volta-migrate  # volta -v
        ) as mock_proc_run:
            install()

            mock_install_volta.assert_called_once_with(f"{tmp_path}/bin")
            assert mock_proc_run.mock_calls == [
                call((f"{tmp_path}/bin/volta-migrate",)),
                call((f"{tmp_path}/bin/volta", "-v"), stdout=True),
            ]
            assert (
                os.readlink(f"{tmp_path}/bin/node")
                == f"{tmp_path}/volta/bin/node"
            )

    # now test already installed
    with patch.multiple(
        "devenv.lib.volta", root=tmp_path, VOLTA_HOME=VOLTA_HOME
    ), patch("devenv.lib.volta.install_volta") as mock_install_volta, patch(
        "devenv.lib.volta.which",
        side_effect=[f"{tmp_path}/bin/volta", f"{tmp_path}/volta/bin/node"],
    ):
        install()
        assert not mock_install_volta.called


def test_install_volta_linux_x86_64() -> None:
    with patch("platform.system", return_value="Linux") as mock_system, patch(
        "platform.machine", return_value="x86_64"
    ) as mock_machine, patch(
        "devenv.lib.volta.archive.download"
    ) as mock_download, patch(
        "devenv.lib.volta.archive.unpack"
    ) as mock_unpack:
        install_volta("/path/to/unpack")
        mock_system.assert_called_once()
        mock_machine.assert_called_once()
        mock_download.assert_called_once_with(
            (
                "https://github.com/volta-cli/volta/releases/download/"
                f"v{_version}/volta-{_version}-linux.tar.gz"
            ),
            _sha256[f"volta-{_version}-linux.tar.gz"],
        )
        mock_unpack.assert_called_once_with(
            mock_download.return_value, "/path/to/unpack"
        )


def test_install_volta_macos_x86_64() -> None:
    with patch("platform.system", return_value="Darwin") as mock_system, patch(
        "platform.machine", return_value="x86_64"
    ) as mock_machine, patch(
        "devenv.lib.volta.archive.download"
    ) as mock_download, patch(
        "devenv.lib.volta.archive.unpack"
    ) as mock_unpack:
        install_volta("/path/to/unpack")
        mock_system.assert_called_once()
        mock_machine.assert_called_once()
        mock_download.assert_called_once_with(
            (
                "https://github.com/volta-cli/volta/releases/download/"
                f"v{_version}/volta-{_version}-macos.tar.gz"
            ),
            _sha256[f"volta-{_version}-macos.tar.gz"],
        )
        mock_unpack.assert_called_once_with(
            mock_download.return_value, "/path/to/unpack"
        )


def test_install_volta_macos_arm64() -> None:
    with patch("platform.system", return_value="Darwin") as mock_system, patch(
        "platform.machine", return_value="arm64"
    ) as mock_machine, patch(
        "devenv.lib.volta.archive.download"
    ) as mock_download, patch(
        "devenv.lib.volta.archive.unpack"
    ) as mock_unpack:
        install_volta("/path/to/unpack")
        mock_system.assert_called_once()
        mock_machine.assert_called_once()
        mock_download.assert_called_once_with(
            (
                "https://github.com/volta-cli/volta/releases/download/"
                f"v{_version}/volta-{_version}-macos-aarch64.tar.gz"
            ),
            _sha256[f"volta-{_version}-macos-aarch64.tar.gz"],
        )
        mock_unpack.assert_called_once_with(
            mock_download.return_value, "/path/to/unpack"
        )


def test_install_volta_unexpected_platform() -> None:
    with patch("platform.system", return_value="Unexpected"):
        with pytest.raises(UnexpectedPlatformError):
            install_volta("/path/to/unpack")


def test_populate_volta_home_with_shims() -> None:
    with patch("devenv.lib.volta.proc.run") as mock_run:
        unpack_into = "/path/to/unpack"
        mock_run.side_effect = [
            None,  # mock run for volta-migrate
            _version,  # mock run for volta -v
        ]

        populate_volta_home_with_shims(unpack_into)

        assert mock_run.mock_calls == [
            call((f"{unpack_into}/volta-migrate",)),
            call((f"{unpack_into}/volta", "-v"), stdout=True),
        ]



from __future__ import annotations

import argparse
import os
import shutil
from collections.abc import Sequence

from typing_extensions import TypeAlias

from devenv.constants import CI
from devenv.constants import DARWIN
from devenv.constants import EXTERNAL_CONTRIBUTOR
from devenv.constants import home
from devenv.constants import homebrew_bin
from devenv.constants import VOLTA_HOME
from devenv.lib import brew
from devenv.lib import direnv
from devenv.lib import github
from devenv.lib import proc
from devenv.lib import volta

help = "Bootstraps the development environment."
ExitCode: TypeAlias = "str | int | None"


def main(coderoot: str, argv: Sequence[str] | None = None) -> ExitCode:
    parser = argparse.ArgumentParser(description=help)
    parser.add_argument(
        "repo",
        type=str,
        nargs="?",
        default="sentry",
        choices=("sentry", "getsentry"),
    )
    args = parser.parse_args(argv)

    if args.repo == "getsentry":
        # Setting up sentry means we're setting up both repos.
        args.repo = "sentry"

    if args.repo not in {"sentry"}:
        return f"repo {args.repo} not supported yet!"

    if not CI and shutil.which("xcrun"):
        # xcode-select --install will take a while,
        # and involves elevated permissions with a GUI,
        # so best to just let the user go through that separately then retrying,
        # rather than waiting for it.
        # There is a way to perform a headless install but it's more complex
        # (refer to how homebrew does it).
        try:
            _ = proc.run(("xcrun", "-f", "git"), stdout=True)
        except RuntimeError:
            return "Failed to find git. Run xcode-select --install, then re-run bootstrap when done."

    github.add_to_known_hosts()

    if not EXTERNAL_CONTRIBUTOR and not github.check_ssh_access():
        is_employee = (
            False
            if CI
            else input("Are you a Sentry employee? (Y/n): ").lower()
            in {"y", "yes", ""}
        )
        if not CI and not is_employee:
            print(
                "Please set the SENTRY_EXTERNAL_CONTRIBUTOR environment variable and re-run bootstrap."
            )
            return 1
        pubkey = github.generate_and_configure_ssh_keypair()
        if not CI:
            input(
                f"""
Failed to authenticate with an ssh key to GitHub.
We've generated and configured one for you at ~/.ssh/sentry-github.
Visit https://github.com/settings/ssh/new and add the following Authentication key:

{pubkey}

Then, you need to go to https://github.com/settings/keys, find your key,
and click Configure SSO, for the getsentry organization.

When done, hit ENTER to continue.
"""
            )
        while not github.check_ssh_access():
            input(
                "Still failing to authenticate to GitHub. ENTER to retry, otherwise ^C to quit."
            )

    brew.install()
    volta.install()
    direnv.install()

    os.makedirs(coderoot, exist_ok=True)

    if args.repo == "sentry":
        if not os.path.exists(f"{coderoot}/sentry"):
            # git@ clones forces the use of cloning through SSH which is what we want,
            # though CI must clone open source repos via https (no git authentication)
            additional_flags = (
                ("--depth", "1", "https://github.com/getsentry/sentry")
                if CI
                else ("git@github.com:getsentry/sentry",)
            )
            proc.run(
                (
                    "git",
                    "-C",
                    coderoot,
                    "clone",
                    # Download all reachable commits and trees while fetching blobs on-demand
                    # https://github.blog/2020-12-21-get-up-to-speed-with-partial-clone-and-shallow-clone/
                    "--filter=blob:none",
                    *additional_flags,
                ),
                exit=True,
            )
        if (
            not CI
            and not EXTERNAL_CONTRIBUTOR
            and not os.path.exists(f"{coderoot}/getsentry")
        ):
            proc.run(
                (
                    "git",
                    "-C",
                    coderoot,
                    "clone",
                    "--filter=blob:none",
                    "git@github.com:getsentry/getsentry",
                ),
                exit=True,
            )

        print("Installing sentry's brew dependencies...")
        if CI:
            if DARWIN:
                # Installing everything from brew takes too much time,
                # and chromedriver cask flakes occasionally. Really all we need to
                # set up the devenv is colima and docker-cli.
                # This is also required for arm64 macOS GHA runners.
                # We manage colima, so just need to install docker + qemu here.
                proc.run(("brew", "install", "docker", "qemu"))
        else:
            proc.run(
                (f"{homebrew_bin}/brew", "bundle"), cwd=f"{coderoot}/sentry"
            )

        # this'll create the virtualenv if it doesn't exist
        proc.run(("devenv", "sync"), cwd=f"{coderoot}/sentry")

        # HACK: devenv sync created the config files earlier, but make bootstrap will
        #       fail because of an interactive prompt asking if user wants to clobber it...
        #       i'll follow-up with fixing that in sentry
        shutil.rmtree(f"{home}/.sentry")

        # make bootstrap should be ported over to devenv sync,
        # as it applies new migrations as well and so would need to ensure
        # the appropriate devservices are running
        proc.run(
            ("make", "bootstrap"),
            env={"VIRTUAL_ENV": f"{coderoot}/sentry/.venv"},
            pathprepend=f"{coderoot}/sentry/.venv/bin",
            cwd=f"{coderoot}/sentry",
        )

        if not CI:
            # HACK: see above
            shutil.rmtree(f"{home}/.sentry")

            # we don't have permissions to clone getsentry which is a good thing
            # eventually we should move this bootstrap testing over to getsentry repo
            proc.run(
                ("make", "bootstrap"),
                env={
                    "VIRTUAL_ENV": f"{coderoot}/getsentry/.venv",
                    "VOLTA_HOME": VOLTA_HOME,
                },
                pathprepend=f"{coderoot}/getsentry/.venv/bin",
                cwd=f"{coderoot}/getsentry",
            )

    print(
        f"""
All done! Please close this terminal window and start a fresh one.

Sentry has been set up in {coderoot}/sentry. cd into it and you should
be able to run `sentry devserver`.
"""
    )
    return 0
